@@ -166,6 +166,8 @@ gnome_bg_slide_show_finalize (GObject *object)
 
         g_list_foreach (self->priv->stack->head, (GFunc) g_free, NULL);
         g_queue_free (self->priv->stack);
+
+        g_free (self->priv->filename);
 }
 
 static void
@@ -682,6 +684,10 @@ parse_file_contents (GnomeBGSlideShow  *self,
 
                 /* no slides, that's not a slideshow */
                 if (len == 0) {
+                        g_set_error_literal (error,
+                                             G_MARKUP_ERROR,
+                                             G_MARKUP_ERROR_INVALID_CONTENT,
+                                             "file is not a slide show since it has no slides");
                         failed = TRUE;
                 /* one slide, there's no transition */
                 } else if (len == 1) {
@@ -772,6 +778,7 @@ gnome_bg_slide_show_load_async (GnomeBGSlideShow    *self,
 
     file = g_file_new_for_path (self->priv->filename);
     g_file_load_contents_async (file, cancellable, (GAsyncReadyCallback) on_file_loaded, task);
+    g_object_unref (file);
 }
 
 /**
