@@ -64,9 +64,6 @@ struct _GnomeDesktopThumbnailFactoryPrivate {
 
 static const char *appname = "gnome-thumbnail-factory";
 
-static void gnome_desktop_thumbnail_factory_init          (GnomeDesktopThumbnailFactory      *factory);
-static void gnome_desktop_thumbnail_factory_class_init    (GnomeDesktopThumbnailFactoryClass *class);
-
 G_DEFINE_TYPE (GnomeDesktopThumbnailFactory,
 	       gnome_desktop_thumbnail_factory,
 	       G_TYPE_OBJECT)
@@ -339,7 +336,7 @@ _gdk_pixbuf_new_from_uri_at_scale (const char *uri,
 {
     gboolean result;
     guchar buffer[LOAD_BUFFER_SIZE];
-    gsize bytes_read;
+    gssize bytes_read;
     GdkPixbufLoader *loader = NULL;
     GdkPixbuf *pixbuf;	
     GdkPixbufAnimation *animation;
@@ -379,7 +376,10 @@ _gdk_pixbuf_new_from_uri_at_scale (const char *uri,
     }
 
     if (input_stream == NULL) {
-        input_stream = gs_file_read_noatime (file, NULL, &error);
+        if (g_file_is_native (file))
+            input_stream = gs_file_read_noatime (file, NULL, &error);
+        else
+            input_stream = G_INPUT_STREAM (g_file_read (file, NULL, &error));
         if (input_stream == NULL) {
             g_warning ("Unable to create an input stream for %s: %s", uri, error->message);
             g_clear_error (&error);
@@ -440,14 +440,18 @@ _gdk_pixbuf_new_from_uri_at_scale (const char *uri,
 	}
     }
 
-    if (gdk_pixbuf_loader_close (loader, &error) == FALSE) {
+    if (loader == NULL) {
+        /* This can happen if the above loop was exited due to the
+         * g_input_stream_read() call failing. */
+        result = FALSE;
+    } else if (gdk_pixbuf_loader_close (loader, &error) == FALSE) {
         g_warning ("Error creating thumbnail for %s: %s", uri, error->message);
         g_clear_error (&error);
         result = FALSE;
     }
 
     if (!result) {
-	g_object_unref (G_OBJECT (loader));
+        g_clear_object (&loader);
 	g_input_stream_close (input_stream, NULL, NULL);
 	g_object_unref (input_stream);
 	g_object_unref (file);
